package com.simuwang.manage.service.competition;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.StrUtil;
import com.simuwang.base.mapper.CompanyInformationMapper;
import com.simuwang.base.mapper.RzZhzq2ResultMapper;
import com.simuwang.base.pojo.dos.CompanyInformationDO;
import com.simuwang.base.pojo.dto.competition.CompetitionBaseResultDTO;
import com.simuwang.base.pojo.dto.competition.ZhzqCompetitionResultDTO;
import org.springframework.stereotype.Service;

import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
public class ZhzqCompetitionResultServiceService extends AbstractCompetitionResultService {

    private static final String GROWTH_PARTNERSHIP_GROUP = "成长伙伴组";

    private static final String STRATEGIC_PARTNERSHIP_GROUP = "战略伙伴组";

    private static final List<Integer> GROWTH_PARTNERSHIP_STRATEGY_LIST = ListUtil.toList(1, 2, 3, 4);

    private static final Map<Integer, String> STRATEGY_ID_NAME_MAP = MapUtil.newHashMap();

    static {
        STRATEGY_ID_NAME_MAP.put(1, "主观多头策略");
        STRATEGY_ID_NAME_MAP.put(2, "股票量化策略");
        STRATEGY_ID_NAME_MAP.put(3, "相对价值策略");
        STRATEGY_ID_NAME_MAP.put(4, "CTA策略");
        STRATEGY_ID_NAME_MAP.put(5, "复合策略");
    }

    private final RzZhzq2ResultMapper zhzqResultMapper;

    private final CompanyInformationMapper companyInformationMapper;

    public ZhzqCompetitionResultServiceService(RzZhzq2ResultMapper zhzqResultMapper, CompanyInformationMapper companyInformationMapper) {
        this.zhzqResultMapper = zhzqResultMapper;
        this.companyInformationMapper = companyInformationMapper;
    }

    @Override
    public boolean isSupport(Integer competitionId) {
        return competitionId != null && competitionId == 49;
    }

    @Override
    protected List<? extends CompetitionBaseResultDTO> getCompetitionResult() {
        List<ZhzqCompetitionResultDTO> competitionResultDTOList = zhzqResultMapper.getResult();
        if (CollUtil.isEmpty(competitionResultDTOList)) {
            return CollUtil.newArrayList();
        }
        // 策略名称和排名
        Map<Integer, List<ZhzqCompetitionResultDTO>> strategyResultDTOList = competitionResultDTOList.stream().collect(Collectors.groupingBy(ZhzqCompetitionResultDTO::getStrategy));
        for (Map.Entry<Integer, List<ZhzqCompetitionResultDTO>> resultEntry : strategyResultDTOList.entrySet()) {
            Integer strategy = resultEntry.getKey();
            String strategyName = STRATEGY_ID_NAME_MAP.get(strategy);
            List<ZhzqCompetitionResultDTO> resultDTOList = resultEntry.getValue();

            // 按照分数降序排序
            resultDTOList.sort(Comparator.comparing(ZhzqCompetitionResultDTO::getScore, Comparator.nullsFirst(Comparator.naturalOrder())).reversed());
            // 计算排名并设置
            int rank = 1;
            for (int i = 0; i < resultDTOList.size(); i++) {
                ZhzqCompetitionResultDTO dto = resultDTOList.get(i);
                dto.setStrategyName(strategyName);
                if (dto.getScore() != null) {
                    dto.setRank(rank);
                    rank = i + 1;
                } else {
                    // 没有分数的基金排名为：999999
                    dto.setRank(999999);
                }
            }
        }

        // 设置规模分组
        settingAssetGroupName(competitionResultDTOList);

        // 策略分组和排名降序输出
        return competitionResultDTOList.stream()
                .sorted(Comparator.comparing(ZhzqCompetitionResultDTO::getStrategyName).reversed().thenComparing(ZhzqCompetitionResultDTO::getRank))
                .collect(Collectors.toList());
    }

    private void settingAssetGroupName(List<ZhzqCompetitionResultDTO> competitionResultDTOList) {
        if (CollUtil.isEmpty(competitionResultDTOList)) {
            return;
        }
        List<String> companyIdList = competitionResultDTOList.stream().map(ZhzqCompetitionResultDTO::getCompanyId).filter(StrUtil::isNotBlank).distinct().toList();
        if (CollUtil.isEmpty(companyIdList)) {
            return;
        }
        // 获取公司规模 -> 1:0-5亿 2:5-10亿 3:10-20亿 4:20-50亿 5:50-100亿 6:100亿以上
        List<CompanyInformationDO> companyInfoDOList = companyInformationMapper.quertAssetByCompanyId(companyIdList);
        Map<String, Integer> companyIdAssetMap = companyInfoDOList.stream().collect(Collectors.toMap(CompanyInformationDO::getCompanyId, CompanyInformationDO::getCompanyAssetSize));

        competitionResultDTOList.forEach(e -> e.setAssetGroupName(getAssetGroupName(companyIdAssetMap.get(e.getCompanyId()), e.getStrategy())));
    }

    private String getAssetGroupName(Integer companyAssetSize, Integer strategyId) {
        // 1:0-5亿 2:5-10亿 3:10-20亿 4:20-50亿 5:50-100亿 6:100亿以上
        if (GROWTH_PARTNERSHIP_STRATEGY_LIST.contains(strategyId)) {
            return companyAssetSize != null && companyAssetSize >= 4 ? STRATEGIC_PARTNERSHIP_GROUP : GROWTH_PARTNERSHIP_GROUP;
        } else {
            return companyAssetSize != null && companyAssetSize >= 3 ? STRATEGIC_PARTNERSHIP_GROUP : GROWTH_PARTNERSHIP_GROUP;
        }
    }
}
